/*
=========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_UPSO_DE.c 
 Usage: Main UPSO and DE subroutines
 Last Modified: 1/12/2010
=========================================================================================================
*/

#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "SOURCE_params.h"
#include "SOURCE_UPSO_DE.h"

//-------------------
// EVALUATE PARTICLE
//-------------------
void Evaluate_Particle (double X[], double *FX, int N, RUN_STATS *stats)
{
	int	i;
	double	Y[N], FY=0.0;
	
	// CREATE CLONE VECTOR TO AVOID CHANGE ORIGINAL X

	for (i=0; i<N; i++)
		Y[i] = X[i];
		
	// EVALUATE CLONE
	Objective_F(Y, N, &FY);
	
	*FX = FY;
	stats->fev = stats->fev + 1;	
}

//=========================================================================================================
//=========================================================================================================

//----------------
// EVALUATE SWARM
//----------------
void Evaluate_Swarm (double swarm[], double fswarm[], int M, int N, RUN_STATS *stats)
{
	int	i, j;
	double X[N], FX;
	
	for (i=0; i<M; i++) 
	{
		// ISOLATE PARTICLE AND CREATE CLONE
		for (j=0; j<N; j++)
			X[j] = swarm[i*N+j];

		Evaluate_Particle(X, &FX, N, &*stats);
		fswarm[i] = FX;

	}
}

//=========================================================================================================
//=========================================================================================================

//-----------------------
// UPDATE BEST POSITIONS
//-----------------------

void Update_Best_Positions (double swarm[], double fswarm[],  double bestpos[], 
	double fbestpos[], int M, int N, RUN_STATS *stats, int *local)
{
	int i, j;
	
	for (i=0; i<M; i++) 
	{	
		if (fswarm[i] < fbestpos[i]) 
		{
			if (norm_diff(N, &swarm[i*N], &bestpos[i*N], 1) > sqrt(XEPS))
				local[i] = 0;

			for (j=0; j<N; j++)
				bestpos[i*N+j] = swarm[i*N+j];
			fbestpos[i] = fswarm[i];
			stats->BPup = stats->BPup + 1;

		}
		
	}
}

//=========================================================================================================
//=========================================================================================================

//----------------------------------
// UPDATE INDICES OF BEST PARTICLES 
//---------------------------------- 
void Update_Best_Indices (double fbestpos[], int g1[], int g2[], ALGO_par algo, RUN_STATS *stats)
{
	int i, j, k;
	
	// LOOP ON PARTICLES
	//-------------------
	for (i=0; i<algo.SS; i++) 
	{
	
		// GLOBAL BEST INDEX g1
		if (fbestpos[i] < stats->bestval) 
		{
			if ( (fabs(fbestpos[i]-stats->bestval)/max(1.0, fabs(stats->bestval))) > FEPS)
			{
				stats->lasthit = stats->iter;
				stats->lastf = stats->fev;
				stats->lastg = stats->gev;
				stats->lastl = stats->localsearch;
			}
			stats->bestval = fbestpos[i];
			stats->bestind = i;
		}
		
		// LOCAL BEST INDEX g2
		g2[i] = i;					
		for (j=(i-algo.nradius); j<=(i+algo.nradius); j++) {	
			k = j;	
			
			// RECYCLE AT START OF SWARM				
			if (j<0)				
				k = algo.SS + j;
			
			// RECYCLE AT END OF SWARM
			else if (j>(algo.SS-1))		
				k = j - algo.SS;
			
			if (fbestpos[k] < fbestpos[g2[i]])		
				g2[i] = k;
		}
	
	}
	
	// UPDATE GLOBAL BEST INDEX G1
	// ----------------------------
	for (i=0; i<algo.SS; i++)
		g1[i] = stats->bestind;

	
}

//=========================================================================================================
//=========================================================================================================

//--------------
// UPDATE SWARM  
//--------------
void Update_Swarm (double swarm[], double fswarm[], double vel[], double bestpos[], double fbestpos[], 
	int g1[], int g2[], ALGO_par algo, PROBLEM_par prob, int algi)
{
	int	i, j, pii, pij;
	double	TA, TB, TC, R;
	int	R1, R2, R3, R4, R5, rnbr;
	int 	*aux;
	double 	g_vel, l_vel;
	aux = (int *) malloc(algo.SS * sizeof(int));
	
	//+++++++++++++++++++++++++++++++++++++++++++
	// UNIFIED PARTICLE SWARM OPTIMIZATION UDATE
	//+++++++++++++++++++++++++++++++++++++++++++
	
	if (algi == 1) {
	
		for (i=0; i<algo.SS; i++) {
			
			pii = i*prob.dim;
			
			for (j=0; j<prob.dim; j++) {
				
				pij = pii + j;
				TA  = algo.c1*(bestpos[pij]-swarm[pij]);
				TB  = algo.c2*(bestpos[g1[i]*prob.dim+j]-swarm[pij]);
				TC  = algo.c2*(bestpos[g2[i]*prob.dim+j]-swarm[pij]);
			
				// GBEST VELOCITY UPDATE
				//-----------------------
				g_vel = algo.chi * ( vel[pij] + drand48()*TA + drand48()*TB );
				
				// LBEST VELOCITY UPDATE
				//-----------------------
				l_vel = algo.chi * ( vel[pij] + drand48()*TA + drand48()*TC );
			
			
				// UPSO VELOCITY WITHOUT MUTATION
				//-----------------------------

				if (algo.R3use == 0)
					vel[pij] = algo.UF * g_vel + (1.0-algo.UF) * l_vel;
			
				// UPSO VELOCITY WITH MUTATION
				//-----------------------------
				else {
					R = normal_distrib();
					R = algo.R3mean + R * algo.R3std;
					
					// MUTATE GLOBAL TERM IF UF<0.5
					//------------------------------
					if (algo.UF < 0.5)
						vel[pij] = algo.UF * R * g_vel + (1.0-algo.UF) * l_vel;
					
					// MUTATE LOCAL TERM IF UF=>0.5
					//------------------------------
					else
						vel[pij] = algo.UF * g_vel + (1.0-algo.UF) * R * l_vel;
				}
			
				// CONSTRAIN VELOCITY WITHIN BOUNDS
				// --------------------------------
				if (vel[pij] > algo.Vmax[j])
					vel[pij] = algo.Vmax[j];
				else if (vel[pij] < -algo.Vmax[j])
					vel[pij] = -algo.Vmax[j];
		
				// UPDATE PARTICLE'S POSITION
				// --------------------------
				swarm[pij] += vel[pij];
			
				// CONSTRAIN PARTICLE WITHIN SEARCH SPACE
				// --------------------------------------
#ifdef TINKER
                		while (swarm[pij] > 180.0)
                        		swarm[pij] = swarm[pij] - 360.0;
        				
                		while (swarm[pij] < -180.0)
                        		swarm[pij] = swarm[pij] + 360.0;
        			
#else
				if (swarm[pij] > prob.Xmax[j])		// Violation of upper bounds
					swarm[pij] = prob.Xmax[j];
				if (swarm[pij] < prob.Xmin[j])		// Violation of lower bounds
					swarm[pij] = prob.Xmin[j];
#endif
			} // END LOOP ON DIMENSION
		
		} // END LOOP ON SWARM SIZE
	
	} // END-IF ON ALGORITHM TYPE
	
       //++++++++++++++++++++++++++++++
       // DIFFERENTIAL EVOLUTION UDATE
       //++++++++++++++++++++++++++++++
       else {

               // REPLICATE DE POPULATION (BESTPOS) TO TEMPORARY SWARM
               //------------------------------------------------------
               copy_vector (algo.SS*prob.dim, bestpos, swarm);
               copy_vector (algo.SS, fbestpos, fswarm);

               for (i=0; i<algo.SS; i++) {

                       pii = i*prob.dim;

                       // GENERATE 5 RANDOM INDICES MUTUALLY DIFFERENT THAN i
                       //-----------------------------------------------------
                       for ( j=0; j<algo.SS; j++ ) aux[j] = j;
                       randperm( algo.SS, aux );
                       j = 0;
                       if ( aux[j] != i ) R1 = aux[j];
                       else {
                               j++;
                               R1 = aux[j];
                       }
                       j++;
                       if ( aux[j] != i ) R2 = aux[j];
                       else {
                               j++;
                               R2 = aux[j];
                       }
                       j++;
                       if ( aux[j] != i ) R3 = aux[j];
                       else {
                               j++;
                               R3 = aux[j];
                       }
                       j++;
                       if ( aux[j] != i ) R4 = aux[j];
                       else {
                               j++;
                               R4 = aux[j];
                       }
                       j++;
                       if ( aux[j] != i ) R5 = aux[j];
                       else {
                               j++;
                               R5 = aux[j];
                       }

                       rnbr = (int) rint(1+drand48()*(prob.dim-1));

                       // LOOP ON DIMENSION
                       //-------------------
                       for (j=0; j<prob.dim; j++) {

                               pij = pii + j;

                               // STRATEGY 1 - DE/BEST/1/BIN
                               //----------------------------
                               if (algo.oper == 1)
                                       vel[pij] = bestpos[g1[i]*prob.dim+j]
                                                 + algo.F * (bestpos[R1*prob.dim+j]-bestpos[R2*prob.dim+j]);

                               // STRATEGY 2 - DE/RAND/1/BIN
                               //----------------------------
                               else if (algo.oper == 2)
                                       vel[pij] = bestpos[R3*prob.dim+j]
                                                 + algo.F * (bestpos[R1*prob.dim+j]-bestpos[R2*prob.dim+j]);

                               // STRATEGY 3 - DE/RAND-TO-BEST/1/BIN
                               //------------------------------------
                               else if (algo.oper == 3)
                                       vel[pij] = bestpos[pij]
                                                 + algo.F * (bestpos[g1[i]*prob.dim+j]-bestpos[pij])
                                                 + algo.F * (bestpos[R1*prob.dim+j]-bestpos[R2*prob.dim+j]);

                               // STRATEGY 4 - DE/BEST/2/BIN
                               //----------------------------
                               else if (algo.oper == 4)
                                       vel[pij] = bestpos[g1[i]*prob.dim+j]
                                                 + algo.F * (bestpos[R1*prob.dim+j]-bestpos[R2*prob.dim+j]
                                                 + bestpos[R3*prob.dim+j]-bestpos[R4*prob.dim+j]);

                               // STRATEGY 5 - DE/RAND/2/BIN
                               //----------------------------
                               else if (algo.oper == 5)
                                       vel[pij] = bestpos[R5*prob.dim+j]
                                                 + algo.F * (bestpos[R1*prob.dim+j]-bestpos[R2*prob.dim+j]
                                                 + bestpos[R3*prob.dim+j]-bestpos[R4*prob.dim+j]);

                               else
                                       printf("DE ERROR: UNKNOWN STRATEGY %i IN DE \n", algo.oper);

                               // GENERATE TRIAL VECTOR
                               //-----------------------
                               if ((drand48() <= algo.CR)||(j == rnbr))
                                       swarm[pij] = vel[pij];
                               else
                                       swarm[pij] = bestpos[pij];

                               // CONSTRAIN INDIVIDUAL WITHIN SEARCH SPACE
                               // -----------------------------------------
#ifdef TINKER
                		while (swarm[pij] > 180.0)
                        		swarm[pij] = swarm[pij] - 360.0;
        				
                		while (swarm[pij] < -180.0)
                        		swarm[pij] = swarm[pij] + 360.0;
#else        			
                               if (swarm[pij] > prob.Xmax[j])          // Violation of upper bounds
                                       swarm[pij] = prob.Xmax[j];
                               if (swarm[pij] < prob.Xmin[j])          // Violation of lower bounds
                                       swarm[pij] = prob.Xmin[j];
#endif

                       } // END LOOP ON DIMENSION

               } // END LOOP ON POPULATION SIZE

       } // END-ELSE ON ALGORITHM TYPE
	
	free(aux);	
}

//=========================================================================================================
//=========================================================================================================

//----------------------------
// POPULATIONS INITIALIZATION  
//----------------------------

void Initialization (double swarm[], double vel[], PROBLEM_par prob, ALGO_par algo)
{
	int i, j;
	
	for (i=0; i<algo.SS; i++) {
		for (j=0; j<prob.dim; j++) {
			swarm[i*prob.dim+j]   = prob.Xmin[j] + drand48()*(prob.Xmax[j]-prob.Xmin[j]);
			if (drand48()<0.5)
				vel[i*prob.dim+j]     = prob.Xmin[j] + drand48()*(swarm[i*prob.dim+j] - prob.Xmin[j]);
			else
				vel[i*prob.dim+j]     = swarm[i*prob.dim+j] + drand48()*(prob.Xmax[j] - swarm[i*prob.dim+j]);
		}
	}
}

void Velocity_Initialization (double swarm[], double vel[], PROBLEM_par prob, ALGO_par algo)
{
	int i, j;
	
	for (i=0; i<algo.SS; i++) {
		for (j=0; j<prob.dim; j++) {
			if (drand48()<0.5)
				vel[i*prob.dim+j]     = prob.Xmin[j] + drand48()*(swarm[i*prob.dim+j] - prob.Xmin[j]);
			else
				vel[i*prob.dim+j]     = swarm[i*prob.dim+j] + drand48()*(prob.Xmax[j] - swarm[i*prob.dim+j]);
		}
	}
}

//=========================================================================================================
//=========================================================================================================
		
void Apply_Memetic(double fbestpos[], double bestpos[], ALGO_par algo, PROBLEM_par prob, RUN_STATS *stats, int *local)
{
	int 	i, k;
	int	STOP=0;
	double 	r;
	double 	*TP, *TPOUT;
	double	VAL, GRMS;
	int 	NF, NG;
		
	TP = (double *) malloc(prob.dim*sizeof(double));
	TPOUT = (double *) malloc(prob.dim*sizeof(double));		
	if (algo.memetic == 2  || algo.memetic  == 3)
	{			
		double r;
		for (i=0; i<algo.SS; i++)
		{
			r = drand48();
			if (r < algo.prob && local[i] != 1)
			{					
				for (k=0; k<prob.dim; k++)
					TP[k] = bestpos[i*prob.dim+k];
				VAL = fbestpos[i];
				local_optimization(TP, prob.dim, TPOUT, &VAL, &GRMS, prob.Xmin, prob.Xmax, &NF, &NG);
				if (GRMS < GEPS)
					local[i] = 1;
				else
					local[i] = 0;
				stats->localsearch = stats->localsearch  + 1;
				stats->fev = stats->fev + NF;		
				stats->gev = stats->gev + NG;		
				for (k=0; k<prob.dim; k++)
					bestpos[i*prob.dim+k] = TPOUT[k];
				fbestpos[i] = VAL;
				
				STOP = StopCriterion(prob, &*stats);
				if (STOP == 1)
					return;
			}
		}				
	}

	if (algo.memetic  == 1 || algo.memetic  == 3)
	{
		if (local[stats->bestind] != 1 )
		{
			for (k=0; k<prob.dim; k++)
			{
				TP[k] = bestpos[stats->bestind*prob.dim+k];
			}	
			VAL = fbestpos[stats->bestind];
			local_optimization(TP, prob.dim, TPOUT, &VAL, &GRMS, prob.Xmin, prob.Xmax, &NF, &NG);
			if (GRMS < GEPS)
				local[stats->bestind] = 1;
			else
				local[stats->bestind] = 0;

			stats->localsearch = stats->localsearch  + 1;
			for (k=0; k<prob.dim; k++)
			{
				bestpos[stats->bestind*prob.dim+k] = TPOUT[k];
			}	
			fbestpos[stats->bestind] = VAL;
			
			stats->bestval = VAL;
			stats->fev = stats->fev + NF;		
		}
	}

	free(TP);
	free(TPOUT);

}		
		
void explode(double swarm[], double fswarm[], double vel[], double bestpos[], double fbestpos[], 
	     ALGO_par algo, PROBLEM_par prob, RUN_STATS *stats)
{
	double sw[prob.dim], fsw, best[prob.dim], fbest;
	int i;

	for (i=0; i<prob.dim; i++)
	{
		best[i] = bestpos[stats->bestind*prob.dim+i];
		sw[i] = swarm[stats->bestind*prob.dim+i];
	}
	fbest = fbestpos[stats->bestind];
	fsw = fswarm[stats->bestind];

	Initialization(swarm, vel, prob, algo);
	Evaluate_Swarm(swarm, fswarm,  algo.SS, prob.dim, stats);
	copy_vector(prob.dim*algo.SS, swarm, bestpos);
	copy_vector(algo.SS, fswarm, fbestpos);
	for (i=0; i<prob.dim; i++)
	{
		bestpos[stats->bestind*prob.dim+i] = best[i];
		swarm[stats->bestind*prob.dim+i] = sw[i];		
	}
	fbestpos[stats->bestind] = fbest;
	fswarm[stats->bestind] = fsw;
	
}
			
void UPSO_DE (double swarm[], double fswarm[], double vel[], double bestpos[], double fbestpos[], 
	 ALGO_par algo, PROBLEM_par prob, RUN_STATS *stats, int algi)
{
	double 		*TP, *TPOUT, std, std_vel;
	int 		*local; 
	double 		VAL, GRMS;
	int 		NF, NG;
	int			STOP=0, i, k, j;
	int*		g1=NULL;
	int*		g2=NULL;
    int 		iter;	
	int			sumi;
	//----------------
	// INITIALIZATION
	//----------------
	g1 = (int *) malloc(algo.SS*sizeof(int));
	g2 = (int *) malloc(algo.SS*sizeof(int));
	local = (int *) malloc(algo.SS*sizeof(int));
	
	TP = (double *) malloc(prob.dim*sizeof(double));
	TPOUT = (double *) malloc(prob.dim*sizeof(double));		
		
	iter	= 0;
	sumi	= 0;
	for (i=0; i<algo.SS; i++)
		local[i] = 0;
	
	if (algo.restart == 0)
	{
		Initialization(swarm, vel, prob, algo);
		Evaluate_Swarm(swarm, fswarm,  algo.SS, prob.dim, stats);
		copy_vector(prob.dim*algo.SS, swarm, bestpos);
		copy_vector(algo.SS, fswarm, fbestpos);
		stats->bestval = fswarm[0];
		stats->bestind = 0;
		Update_Best_Indices(fbestpos, g1, g2, algo, stats);
	}
	else
	{
		if (algo.new_vel == 1)
			 Velocity_Initialization(swarm, vel, prob, algo);

		Update_Best_Positions(swarm, fswarm, bestpos, fbestpos, algo.SS, prob.dim, stats, local);
		Update_Best_Indices(fbestpos, g1, g2, algo,  stats);
	}
	
	
	//---------------------
	// MAIN ITERATION LOOP
	//---------------------
	sumi = 0;
	while (STOP == 0) 
	{		
		stats->iter = stats->iter + 1;
		Update_Swarm(swarm, fswarm, vel, bestpos, fbestpos, g1, g2, algo, prob, algi);
		Evaluate_Swarm(swarm, fswarm, algo.SS, prob.dim, &*stats);
		Update_Best_Positions(swarm, fswarm, bestpos, fbestpos, algo.SS, prob.dim, &*stats, local);
		Update_Best_Indices(fbestpos, g1, g2, algo, stats);

		Apply_Memetic(fbestpos, bestpos, algo, prob, &*stats, local);

		Update_Best_Indices(fbestpos, g1, g2, algo, stats);
		//--------------------------
		// CHECK STOPPING CRITERION
		//--------------------------
		STOP = StopCriterion(prob, &*stats);

		//-------------------------
		// UPDATE ITERATION COUNTER
		//-------------------------
		iter = iter + 1; 

		for (i=0; i<prob.dim; i++)
			stats->best_x[i] = bestpos[stats->bestind*prob.dim+i];
		stats->bestval = fbestpos[stats->bestind];

		sumi = 0;
		for (i=0; i<algo.SS; i++)
			sumi+=local[i];

		if (sumi == algo.SS)
		{
			// Ola exoun local minimum
			//printf("<<< Exploding >>> \n");
			explode(swarm, fswarm, vel, bestpos, fbestpos, algo, prob, stats);
			Update_Best_Indices(fbestpos, g1, g2, algo, stats);
			for (i=0; i<algo.SS; i++)
				local[i] = 0;
		}

		//-------------------------
		// WRITE TEMP RESULTS
		//-------------------------
		if (algo.dump > 0)
		{
			if (iter%algo.dump == 0 )
			{
				if (algo.algi == 1)
				{
					std =  euclidean_std(prob.dim, algo.SS, swarm);
					std_vel = (mat_max(prob.dim, algo.SS, vel));
					printf("Iter: %5d, FunEvals: %10ld, Val: %E, Std: %f, Vel: %f \n", iter, stats->fev, stats->bestval, std, std_vel); 
					fflush(stdout);
				
				}
				else
				{
					std =  mat_stdev (prob.dim, algo.SS, bestpos);
					printf("Iter: %5d, FunEvals: %10ld, Val: %E, Std: %f \n", iter, stats->fev, stats->bestval, std); 
					fflush(stdout);
				}	
				/*printf("X = [ "); 
				for (i=0; i<prob.dim; i++)
					printf(" %f ", stats->best_x[i]);
				printf("\n");
				*/
				
				if (noprint == 1)
				{			
					fprintf(algo.tmpfile, "Iter: %d, FunEvals: %ld, Val: %lf, Sol:[] \n", iter, stats->fev, stats->bestval); 
					fflush(algo.tmpfile);
				}
			}
		}
	}
   //		print_swarm(swarm, prob.dim, algo.SS);
//		print_swarm(vel, prob.dim, algo.SS);
	
	// CHECK SUCCESS
	//---------------
	if ( prob.iVTR != -1 && fabs(stats->bestval - prob.VTR) / max(1.0, fabs(stats->bestval)) < FEPS)
		stats->success = 1;
	else
		stats->success = 0;
	
	free(g1);
	free(g2);
	free(TP);
	free(TPOUT);
	free(local);

}

