/*
=========================================================================================================
 C. Voglis, Department of Computer Science, 
 	    Univeristy of Ioannina, Greece
 	    
 File: SOURCE_params.h 
 Usage: Header file containing defintions of structures, as well as the array 
        of the long options.
 Last Modified: 1/12/2010
=========================================================================================================
*/

/*
=========================================

 PARAMETER STRUCTURES
 
========================================= 
*/
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#define INF 1.0e+300
/*
 *
  ALGORITHM PARAMETERS
 *
 */
typedef struct ALGO_par1 
{
	int 	algi; 		/* Which algorithm: PSO or DE */
	double	chi;		/* PSO: constriction factor */
	double	c1;		/* PSO: personal best factor */
	double	c2;		/* PSO: "global" best factor */
	int	nradius;	/* PSO: the neighborhood of the particle 
				        assuming a ring topology*/
	double	UF;		/* PSO: unification factor */
	int	R3use;		/* PSO: flag indicating the application of mutation */
	double	R3mean;		/* PSO: mean of the normal distribution for the mutation */
	double	R3std;		/* PSO: standard deviation of the normal distribution for the mutation*/
	double	*Vmax;		/* PSO: maximum velocity (per particle) */
	int 	memetic;	/* Memetic: Defines the strategy for the application of local search */
	int 	stopping;	/* Which stopping criterion should be used */
	double  Vscale;		/* PSO: scaling factor for the initialization of the velocity  */
	double  prob;		/* Memetic: probability for starting local search (strategy 2, 3) */
	int 	dump;		/* PSO: dump temporary results every 'dump' iterations */
	//-------------
	double	F;		/* DE: */
	double	CR;		/* DE: */
	int	oper;		/* DE: Switches between different DE strategies */
	//-------------
	int	SS;		/* Swarm size */
	//
	int 	seed;		/* Random seed */
	FILE	*tmpfile;	/* Temporary results file descriptor */
	int	restart;	/* 0: Random initial population, 1: read initial population from file*/
	int 	dump_swarm;
	int 	new_vel;
	
}ALGO_par;

/*
 *
   PROBLEM PARAMETERS
 *
 */
typedef struct PROBLEM_par1 
{
	int	aa;		/* Objective function id */
	int	dim;		/* Objective function dimensionality */
	int	nexp;		/* Number of experiments (different seed) */
	long	maxit;		/* Maximum number of iterations (cycles) */
	long	maxfev;		/* Maximum number of function evaluations */
	long	maxgev;		/* Maximum number of gradient evaluations */
	double	VTR;		/* Objective function's value at the global minimum */
	int 	iVTR;		/* Use target value as termination criterion */
	double*	Xmin;		/* Problem's lower bounds */
	double*	Xmax;		/* Problem's upper bounds */
}PROBLEM_par;

/*
 *
  RUN STATISTICS/experiment
 *   
 */
typedef struct RUN_STATS1 
{
	int	success;	/* Indicates whether the global minimum value was reached */
	long	iter;		/* Total number of iterations (cycles) performed */
	long	fev;		/* Total number of function evaluation used */
	long	BPup;		/* Total number of best position updates */
	long	lasthit;	/* The last iteration a best position was attained */
	int	bestind;	/* The index of the "best" particle */
	double	bestval;	/* Objective function value of the best particle */
	double	cpu;		/* Total wall time */
	int 	localsearch;	/* Total number of local searches */
	int 	gev;		/* Total number of gradient evaluations */
	double 	*best_x;
	double  best_f;
	long	lastf;
	long	lastg;
	long	lastl;
	
}RUN_STATS;

/*
 *
  Structure long_options holds the correspondace between 
  the short (-a) and the long (--algorithm) flags of the 
  command line.
 *
 */
static struct option long_options[] =
{
               /* These options set a flag. */
               {"help", no_argument,   0, 'h'},
               {"file-output",  no_argument,  0, 'p'},
               {"print-solution",    no_argument, 0, 'x'},
               /* These options don't set a flag.
                  We distinguish them by their indices. */
               {"algorithm",  required_argument,     0, 'a'},
               {"objective",  required_argument,     0, 'o'},
               {"dimension",  required_argument, 0, 'd'},
               {"swarm-size",  required_argument, 0, 's'},
               {"max-fun-evals",    required_argument, 0, 'f'},
               {"max-grad-evals",    required_argument, 0, 'g'},
               {"max-iter",    required_argument, 0, 'i'},
               {"target-value",    required_argument, 0, 't'},
               {"num-experiments",    required_argument, 0, 'e'},
               {"radius",    required_argument, 0, 'r'},
               {"unification-factor",    required_argument, 0, 'u'},
               {"max-fun-evals",    required_argument, 0, 'f'},
               {"pso-params",    required_argument, 0, 'b'},
               {"seed",    required_argument, 0, 'n'},
               {"use-mutation",    required_argument, 0, 'U'},
               {"mutation-mean",    required_argument, 0, 'M'},
               {"mutation-std",    required_argument, 0, 'S'},
               {"F-param",    required_argument, 0, 'F'},               
	       {"CR-param",    required_argument, 0, 'C'},
               {"de-strategy",    required_argument, 0, 'v'},	
               {"out-file-prefix",    required_argument, 0, 'O'},
               {"memetic",    required_argument, 0, 'l'},
               {"velocity-scaling",    required_argument, 0, 'c'},
               {"prob-local-search",    required_argument, 0, 'P'},
               {"dump-iterations",    required_argument, 0, 'D'},
               {"EPS",    required_argument, 0, 'E'},
               {"XEPS",    required_argument, 0, 'X'},
               {"FEPS",    required_argument, 0, 'Y'},             
               {"GEPS",    required_argument, 0, 'G'},             
               {"infile",    required_argument, 0, 'y'},
               {"outfile",    required_argument, 0, 'z'},   			
               {"dump-to-file",    required_argument, 0, 'W'},
               {"restore-from-file",    required_argument, 0, 'R'},   
               {0, 0, 0, 0}
};

/*
 *
  Various definitions: absolute value, maximum and minimum
 *
 */
#define abss(a)     (a<0 ? (-a) : a)
#define max(a , b)  (a>b ?   a  : b)
#define min(a , b)  (a>b ?   b  : a)

double 		EPS, FEPS, XEPS, GEPS;
char 		TMPIN[100], TMPOUT[100];
char 		TMPINFILE[20];
char 		TMPOUFILE[20];
ALGO_par        algo;
PROBLEM_par     prob;
RUN_STATS       stats;
double          * swarm;
double          * bestpos;
double          * vel;
double          * fswarm;
double          * fbestpos;


